"""paicli: A CLI tool for PAI (Platform for AI).

Author: Sotetsu KOYAMADA
"""
import json
import requests
from .utils import to_str


class API(object):

    """API client for PAI

    See: https://github.com/Microsoft/pai/blob/master/rest-server/README.md
    """

    def __init__(self, config):
        self.config = config

    def post_token(self, username, password, expiration=500000):
        url = "{}/api/{}/token".format(self.config.api_uri, self.config.api_version)
        headers = {"Content-type": "application/json"}
        data = json.dumps({
            "username": username,
            "password": password,
            "expiration": expiration
        })
        res = requests.post(url, headers=headers, data=data)

        if res.ok:
            return to_str(res.content)
        else:
            res.raise_for_status()

    def put_user(self):
        pass

    def delete_user(self):
        """Admin only."""
        pass

    def put_user_username_virtualclusters(self):
        """Admin only."""
        pass

    def get_jobs(self, username=""):
        url = "{}/api/{}/jobs".format(self.config.api_uri, self.config.api_version)

        params = {} if not username else {"username": username}
        res = requests.get(url=url, params=params)

        if res.ok:
            return to_str(res.content)
        else:
            res.raise_for_status()

    def get_user_username_jobs_jobname(self, username, jobname):
        url = "{}/api/{}/user/{}/jobs/{}".format(
            self.config.api_uri, self.config.api_version, username, jobname
        )
        res = requests.get(url)

        if res.ok:
            return to_str(res.content)
        else:
            res.raise_for_status()

    def get_jobs_jobname_config(self, jobname):
        pass

    def get_user_username_jobs_jobname_ssh(self, username, jobname):
        url = "{}/api/{}/user/{}/jobs/{}/ssh".format(
            self.config.api_uri, self.config.api_version, username, jobname
        )
        res = requests.get(url)

        if res.ok:
            return to_str(res.content)
        else:
            res.raise_for_status()

    def post_user_username_jobs(self, username, job_config_json):
        url = "{}/api/{}/user/{}/jobs".format(self.config.api_uri, self.config.api_version, username)
        headers = self._headers_with_auth()

        res = requests.post(url, headers=headers, data=job_config_json)

        if res.ok:
            return to_str(res.content)
        else:
            res.raise_for_status()

    def get_user_username_jobs_jobname_ssh(self, username, jobname):
        url = "{}/api/{}/user/{}/jobs/{}/ssh".format(self.config.api_uri, self.config.api_version, username, jobname)
        res = requests.get(url)

        if res.ok:
            return to_str(res.content)
        else:
            res.raise_for_status()

    def put_user_username_jobs_jobname_executiontype(self, username, jobname, value):
        url = "{}/api/{}/user/{}/jobs/{}/executionType".format(
            self.config.api_uri, self.config.api_version, username, jobname
        )
        headers = self._headers_with_auth()
        data = json.dumps({"value": value})

        res = requests.put(url, headers=headers, data=data)
        if res.ok:
            return to_str(res.content)
        else:
            res.raise_for_status()

    def get_virtualclusters(self):
        pass

    def get_virtualclusters_vcname(self, vcname):
        pass

    def _headers_with_auth(self):
        self.config.load_access_token()
        headers = {
            "Authorization": "Bearer {}".format(self.config.access_token),
            "Content-type": "application/json"
        }
        return headers



"""paicli: A CLI tool for PAI (Platform for AI).

Author: Sotetsu KOYAMADA
"""
from __future__ import division
import sys
from datetime import datetime
import json
from prettytable import PrettyTable
from termcolor import colored
from .utils import to_str

if sys.platform.lower()[:3] == "win":
    import colorama
    colorama.init()


class Jobs(object):

    def __init__(self, api, username=""):

        self._api = api
        self._jobs = json.loads(self._api.get_jobs(username))
        for job in self._jobs:
            job["createdTime"] = datetime.fromtimestamp(int(job["createdTime"]) / 1000).strftime('%Y-%m-%d %H:%M:%S')

    def __getitem__(self, item):
        return self._jobs[item]

    def __len__(self):
        return len(self._jobs)

    def filter(self, dic):
        """Filter jobs

        EXAMPLE
        -------
        >>> jobs = Jobs(api)
        >>> jobs.filter({"state", ["RUNNING", "SUCCEEDED"]})

        :param dic: dictionary of (str, list)
        """
        _content = []
        for job in self._jobs:
            for key, val in dic.items():
                if job[key] not in val:
                    break
            else:
                _content.append(job)

        self._jobs = _content

    def show(self, n=None):
        tab = PrettyTable()
        tab.field_names = ["name", "username", "virtual cluster", "created time", "retries", "state"]
        for i, line in enumerate(self):
            state = line["state"]
            if state == "SUCCEEDED":
                state = colored(state, "green")
            elif state == "RUNNING":
                state = colored(state, "cyan")
            elif state == "FAILED":
                state = colored(state, "red")

            tab.add_row([line["name"], line["username"], line["virtualCluster"],
                         line["createdTime"], line["retries"], state])

            if n is not None and i + 1 >= n:
                break

        tab.border = False
        print(to_str(tab.get_string()))



import logging
logger = logging.getLogger(__name__)


def to_str(inp):
    _type = type(inp)
    try:
        unicode # python2
        inp = inp.decode('utf-8')
    except NameError:
        try:
            inp = str(inp, 'utf-8')
        except TypeError:
            pass

    logger.debug("Casted \"{}\" from {} to {}".format(inp, _type, type(inp)))

    return inp



"""paicli: A CLI tool for PAI (Platform for AI).

Author: Sotetsu KOYAMADA
"""
from __future__ import print_function
import sys
import click
import logging
import json
from termcolor import colored
import requests
from prettytable import PrettyTable

from .config import Config
from .jobs import Jobs
from .ssh import run_ssh
from .intereactive import select_choices_interactively
from .api import API
from .cmd import token


try:
    FileNotFoundError
except NameError:
    FileNotFoundError = IOError

if sys.platform.lower()[:3] == "win":
    import colorama
    colorama.init()


def _load(config):
    try:
        config.load()
    except FileNotFoundError as e:
        print("Config file does not exist. Run 'pai config'")
        exit(1)


@click.group(help="A CLI tool for PAI (Platform for AI).")
def main():
    pass


@click.command('config', help="Add a your configuration in $HOME/.paicli")
@click.option('--profile', '-p', type=str, default='default', help="Add another profile configuration.")
def configcmd(profile):
    config = Config(profile)
    try:
        config.load()
    except FileNotFoundError as e:
        pass  # No config file
    except KeyError as e:
        pass  # No same profile

    config.add_profile()  # Anyway, add new profile or modify the profile


@click.command('token', help="Update access token.")
@click.option('--expiration', '-e', type=int, default=500000, help="Expiration time.")
@click.option('--profile', '-p', type=str, default='default', help="Use a specified profile.")
def tokencmd(expiration, profile):
    config = Config(profile)
    _load(config)
    api = API(config)

    try:
        token(api, expiration)
    except requests.HTTPError as e:
        print(colored("Failed to update access token.\n", "red"))
        print(e)
        exit(1)


@click.command(name='ssh', help="SSH into a running container in PAI.")
@click.argument('jobname', type=str, default='')
@click.option('--task-name', '-t', type=str, default='')
@click.option('--task-index', '-i', type=int, default=-1)
@click.option('--username', '-u', type=str, default='')
@click.option('--command', '-c', type=str, default='')
@click.option('--dryrun', '-d', is_flag=True)
@click.option('--profile', '-p', type=str, default='default', help="Use a specified profile.")
def sshcmd(jobname, task_name, task_index, username, command, dryrun, profile):
    config = Config(profile)
    _load(config)
    api = API(config)
    if not username:
        username = config.username

    if not jobname:
        jobs = Jobs(api, username)
        jobs.filter({'state': ['RUNNING']})
        if len(jobs) == 0:
            print("There is no running jobs.")
            exit(1)
        choices = [job['name'] for job in jobs]
        _jobname = select_choices_interactively(choices)
    else:
        _jobname = jobname

    content = None
    try:
        content = json.loads(api.get_user_username_jobs_jobname_ssh(username, _jobname))
    except requests.HTTPError as e:
        try:
            content = json.loads(api.get_user_username_jobs_jobname_ssh(config.username, _jobname))
        except requests.HTTPError as e:
            status_code = e.response.status_code
            if status_code == 404:
                print(colored("SSH failed.", "red"))
                if jobname:
                    print("Wrong job name or SSH is not ready yet.\n")
                else:
                    print("SSH is not ready yet. Wait a minute and try again.\n")
            print(e)
            exit(1)

    try:
        run_ssh(api, username, _jobname, task_name, task_index, config, content, command, dryrun)
    except KeyError:  # TODO: raise/catch original error
        print(colored("SSH failed.", "red"))
        print("There is no match task.\n")  # TODO: give more information
        exit(1)


@click.command(name='jobs', help="Show jobs in PAI.")
@click.option('--username', '-u', multiple=True)
@click.option('--state', '-s', multiple=True)
@click.option('--num-jobs', '-n', type=int, default=20)
@click.option('--profile', '-p', type=str, default='default', help="Use a specified profile.")
def jobscmd(username, state, num_jobs, profile):
    config = Config(profile)
    _load(config)
    api = API(config)

    jobs = Jobs(api)
    filter_dic = {}
    if len(username) != 0:
        filter_dic['username'] = username
    if len(state) != 0:
        filter_dic['state'] = state
    if filter_dic:
        jobs.filter(filter_dic)
    jobs.show(num_jobs)


@click.command(name='submit',
               help="Submit your job into PAI. With no json files, or when '-' is specified, read standard input.")
@click.argument('job_config_json', nargs=-1, required=False)
@click.option('--profile', '-p', type=str, default='default', help="Use a specified profile.")
def submitcmd(job_config_json, profile):
    config = Config(profile)
    _load(config)
    api = API(config)

    job_config_json_list = []
    if job_config_json and job_config_json[0] != '-':
        for _job_config_json in job_config_json:
            with open(_job_config_json, 'r') as f:
                job_config_json_list.append(''.join([line.strip('\n').strip() for line in f.readlines()]))
    else:
        stdin_json = ""
        for line in sys.stdin:
            stdin_json += line.strip('\n').strip()
        job_config_json_list.append(stdin_json)

    for _job_config_json in job_config_json_list:
        try:
            api.post_user_username_jobs(config.username, _job_config_json)
            print(colored("Successfully submitted!", "green") + ": {}"
                  .format(json.loads(_job_config_json)['jobName']))
        except requests.HTTPError as e:
            status_code = e.response.status_code
            if status_code == 401:
                print(colored("Submission failed.", "red"))
                print("Access token seems to be expired.")
                print("Update your token by 'pai token', then try again.\n")
            elif status_code == 400:
                print(colored("Submission failed.", "red"))
                print("This may be caused by duplicated submission.\n")
            else:
                print(colored("Submission failed.\n", "red"))
            print(e)
            exit(1)
        except requests.Timeout as e:
            print(colored("Submission failed.\n", "red"))
            print(e)
            exit(1)
        except requests.ConnectionError as e:
            print(colored("Submission failed.\n", "red"))
            print(e)
            exit(1)
        except requests.RequestException as e:
            print(colored("Submission failed.\n", "red"))
            print(e)
            exit(1)
        except FileNotFoundError as e:
            print(colored("Submission failed.\n", "red"))
            print("Access token does not exist. Run 'pai token'")
            exit(1)


@click.command(name='stop', help="Stop a job in PAI.")
@click.argument('jobname', type=str, nargs=-1)
@click.option('--profile', '-p', type=str, default='default', help="Use a specified profile.")
def stopcmd(jobname, profile):
    config = Config(profile)
    _load(config)
    api = API(config)

    if not jobname:
        jobs = Jobs(api, config.username)
        jobs.filter({'state': ['RUNNING']})
        choices = [job['name'] for job in jobs]
        jobname = (select_choices_interactively(choices), )

    for _jobname in jobname:
        try:
            api.put_user_username_jobs_jobname_executiontype(config.username, _jobname, "STOP")
            print(colored("Stop signal submitted!", "green") + ": {}".format(_jobname))
        except requests.HTTPError as e:
            print(colored("Failed to submit a stop signal.\n", "red"))
            print(e)
            exit(1)
        except requests.Timeout as e:
            print(colored("Failed to submit a stop signal.\n", "red"))
            print(e)
            exit(1)
        except requests.ConnectionError as e:
            print(colored("Failed to submit a stop signal.\n", "red"))
            print(e)
            exit(1)
        except requests.RequestException as e:
            print(colored("Failed to submit a stop signal.\n", "red"))
            print(e)
            exit(1)
        except FileNotFoundError:
            print(colored("Failed to submit a stop signal.\n", "red"))
            print("Access token does not exist. Run 'pai token'")
            exit(1)


@click.command(name='host', help="Show host information of a job.")
@click.argument('jobname', type=str)
@click.option('--username', '-u', type=str, default='')
@click.option('--profile', '-p', type=str, default='default', help="Use a specified profile.")
def hostcmd(jobname, username, profile):
    config = Config(profile)
    _load(config)
    api = API(config)
    if not username:
        username = config.username

    tab = PrettyTable()
    tab.field_names = ["task role", "ip", "port label", "port"]

    try:
        ret = api.get_user_username_jobs_jobname(username, jobname)
        tasks = json.loads(ret)['taskRoles']
        for k, v in tasks.items():
            task_name = k
            tasks_statuses = v['taskStatuses']
            for task_status in tasks_statuses:
                container_ip = task_status['containerIp']
                container_ports = task_status['containerPorts']
                for port_label, port in container_ports.items():
                    tab.add_row([task_name, container_ip, port_label, port])

        tab.border = False
        print(tab.get_string())

    except requests.HTTPError as e:
        print(colored("Failed to get a host information.\n", "red"))
        print(e)
        exit(1)
    except requests.Timeout as e:
        print(colored("Failed to get a host information.\n", "red"))
        print(e)
        exit(1)
    except requests.ConnectionError as e:
        print(colored("Failed to get a host information.\n", "red"))
        print(e)
        exit(1)
    except requests.RequestException as e:
        print(colored("Failed to get a host information.\n", "red"))
        print(e)
        exit(1)


main.add_command(configcmd)
main.add_command(tokencmd)
main.add_command(sshcmd)
main.add_command(jobscmd)
main.add_command(submitcmd)
main.add_command(stopcmd)
main.add_command(hostcmd)


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO,
                        format='[%(asctime)s] %(module)s.%(funcName)s %(levelname)s \t: %(message)s')
    logging.debug("Program start.")
    main()



"""paicli: A CLI tool for PAI (Platform for AI).

Author: Sotetsu KOYAMADA
"""
from __future__ import unicode_literals
from prompt_toolkit.application import Application
from prompt_toolkit.interface import CommandLineInterface
from prompt_toolkit.key_binding.manager import KeyBindingManager
from prompt_toolkit.keys import Keys
from prompt_toolkit.layout.containers import Window
from prompt_toolkit.shortcuts import create_eventloop
from prompt_toolkit.filters import IsDone
from prompt_toolkit.layout.controls import TokenListControl
from prompt_toolkit.layout.containers import ConditionalContainer, ScrollOffsets, HSplit
from prompt_toolkit.layout.dimension import LayoutDimension as D
from prompt_toolkit.token import Token
from prompt_toolkit.styles import style_from_dict

# Reference
#
# examples/custom-token-list-control.py:
# https://github.com/jonathanslenders/python-prompt-toolkit/pull/427/commits/2b75d1835eee49c8881c1d8d7e107237227eda95#diff-a4228333fe0e9eca1e3149e15ae2d8de
#


def select_choices_interactively(choices):
    # choices = [job['name'] for job in job_content]
    app = get_app(choices)

    eventloop = create_eventloop()
    try:
        cli = CommandLineInterface(application=app, eventloop=eventloop)
        selected_choice = cli.run(reset_current_buffer=False)
    finally:
        eventloop.close()

    return selected_choice


def get_app(choices):

    class InquirerControl(TokenListControl):
        selected_option_index = 0
        answered = False
        choices = []
    
        def __init__(self, choices, **kwargs):
            self.choices = choices
            super(InquirerControl, self).__init__(self._get_choice_tokens, **kwargs)
    
        @property
        def choice_count(self):
            return len(self.choices)
    
        def _get_choice_tokens(self, cli):
            tokens = []
            T = Token
    
            def append(index, label):
                selected = (index == self.selected_option_index)
                tokens.append((T.Selected if selected else T, '> ' if selected else '  '))
                if selected:
                    tokens.append((Token.SetCursorPosition, ''))
                tokens.append((T.Selected if selected else T, '%-24s' % label))
                tokens.append((T, '\n'))
    
            for i, choice in enumerate(self.choices):
                append(i, choice)
            tokens.pop()  # Remove last newline.
            return tokens
    
        def get_selection(self):
            return self.choices[self.selected_option_index]

    ic = InquirerControl(choices)

    def get_prompt_tokens(cli):
        tokens = []
        if ic.answered:
            cli.return_value = lambda: ic.get_selection()
        return tokens
    
    layout = HSplit([
        Window(height=D.exact(0),
               content=TokenListControl(get_prompt_tokens, align_center=False)),
        ConditionalContainer(
            Window(
                ic,
                width=D.exact(43),
                height=D(min=3),
                scroll_offsets=ScrollOffsets(top=1, bottom=1)
            ),
            filter=~IsDone())])
    
    manager = KeyBindingManager.for_prompt()
    
    @manager.registry.add_binding(Keys.ControlQ, eager=True)
    @manager.registry.add_binding(Keys.ControlC, eager=True)
    def _(event):
        event.cli.set_return_value(None)
    
    @manager.registry.add_binding(Keys.Down, eager=True)
    @manager.registry.add_binding(Keys.ControlN, eager=True)
    def move_cursor_down(event):
        ic.selected_option_index = (
                (ic.selected_option_index + 1) % ic.choice_count)
    
    @manager.registry.add_binding(Keys.Up, eager=True)
    @manager.registry.add_binding(Keys.ControlP, eager=True)
    def move_cursor_up(event):
        ic.selected_option_index = (
                (ic.selected_option_index - 1) % ic.choice_count)
    
    @manager.registry.add_binding(Keys.Enter, eager=True)
    def set_answer(event):
        ic.answered = True
        event.cli.set_return_value(None)
    
    inquirer_style = style_from_dict({
        Token.QuestionMark: '#5F819D',
        Token.Selected: '#FF9D00',
        Token.Instruction: '',
        Token.Answer: '#FF9D00 bold',
        Token.Question: 'bold',
    })
    
    app = Application(
        layout=layout,
        key_bindings_registry=manager.registry,
        mouse_support=True,
        style=inquirer_style
    )

    return app





"""paicli: A CLI tool for PAI (Platform for AI).

Author: Sotetsu KOYAMADA
"""
from __future__ import print_function

import os
import requests
import subprocess
import codecs
import json
from .utils import to_str
from .intereactive import select_choices_interactively


def run_ssh(api, username, jobname, task_name, task_index, config, content, command="", dryrun=False):
    sshkey = _download_sshkey(content)
    path_to_sshkey = os.path.join(config.path_to_configdir, ".tmpkey")

    host = ""
    port = -1

    # if the job has only one task
    if len(content["containers"]) == 1:
        host = content["containers"][0]["sshIp"]
        port = content["containers"][0]["sshPort"]
    # if the job has >= 2 tasks
    else:
        choices = []
        ret = api.get_user_username_jobs_jobname(username, jobname) # TODO: Error handling
        tasks = json.loads(ret)['taskRoles']
        for k, v in tasks.items():
            _task_name = k
            if task_name and _task_name != task_name:
                continue
            tasks_statuses = v['taskStatuses']
            for task_status in tasks_statuses:
                _task_index = task_status['taskIndex']
                if task_index != -1 and task_index != _task_index:
                    continue
                container_id = task_status['containerId']
                container_ip = task_status['containerIp']
                choices.append(
                    (to_str("{} [{}] {}".format(_task_name, _task_index, container_ip)), container_id)
                )

        if not choices:
            raise KeyError  # TODO: raise original Error (No such job/task/task_index)

        if len(choices) == 1:
            _id = choices[0][1]
        else:
            selected = select_choices_interactively([c[0] for c in choices])
            _id = dict(choices)[selected]

        # select unique host/port from given container id
        for c in content["containers"]:
            if c["id"] == _id:
                host = c["sshIp"]
                port = c["sshPort"]
                break

    if os.path.exists(path_to_sshkey):
        os.remove(path_to_sshkey)

    with codecs.open(path_to_sshkey, 'w', 'utf-8') as f:
        f.writelines(sshkey)
    os.chmod(path_to_sshkey, 0o600)

    cmd = ["ssh", "-i", path_to_sshkey, "-p", port, "-oStrictHostKeyChecking=no", "root@{}".format(host)]
    if command:
        cmd.append(command)

    if dryrun:
        print(' '.join(cmd))
    else:
        try:
            subprocess.call(cmd)
        finally:
            if os.path.exists(path_to_sshkey):
                os.remove(path_to_sshkey)


def _download_sshkey(content):
    res = requests.get(content['keyPair']['privateKeyDirectDownloadLink'])
    sshkey = res.content
    return to_str(sshkey)



import json
import getpass

from .utils import to_str


def token(api, expiration):
    password = ""
    if api.config.password:
        password = api.config.password
    else:
        password = to_str(getpass.getpass("Enter password:\n"))
    ret = api.post_token(api.config.username, password, expiration)
    token = json.loads(ret)['token']
    api.config.access_token = token
    api.config.write_access_token()



"""paicli: A CLI tool for PAI (Platform for AI).

Author: Sotetsu KOYAMADA
"""
from __future__ import print_function
import os
import sys
import yaml
import codecs
from .utils import to_str

if sys.version_info[0] == 2:
    input = raw_input

try:
    FileNotFoundError
except NameError:
    FileNotFoundError = IOError

HOME = os.path.expanduser("~")


class Config(object):

    # consts
    _paicli_dirname = ".paicli"
    _configfile = "config.yaml"
    _accesstoken = ".token_{}"

    # paths
    path_to_configdir = os.path.join(HOME, _paicli_dirname)
    path_to_configfile = os.path.join(HOME, _paicli_dirname, _configfile)

    def __init__(self, profile="default"):
        self.profile = to_str(profile)
        self.host = to_str("10.0.3.9")  # default
        self._port = 9186  # default
        self.username = to_str("")  # default
        self.password = to_str("")  # default
        self.access_token = to_str("")
        self.api_version = to_str("v1")


        self.path_to_accesstoken = os.path.join(HOME,
                                                self._paicli_dirname, self._accesstoken.format(self.profile))

    def add_profile(self):
        self.host = to_str(input("Host ip address for PAI [{}]: ".format(self.host)) or self.host)
        self.port = input("Port for REST API [{}]: ".format(self.port)) or self.port
        self.username = to_str(input("Your username [{}]: ".format(self.username)) or self.username)
        self.password = to_str(input("Your password (optional) [{}]: ".format(self.password)) or self.password)
        self.write_config()

    def load(self):
        with codecs.open(self.path_to_configfile, 'r', 'utf-8') as f:
            yaml_config = ''.join(f.readlines())
            config = yaml.load(yaml_config)

        self.host = config[self.profile]["host"]
        self.port = config[self.profile]["port"]
        self.port = config[self.profile]["port"]
        self.username = config[self.profile]["username"]
        self.password = config[self.profile]["password"]

    def load_access_token(self):
        with codecs.open(self.path_to_accesstoken, 'r', 'utf-8') as f:
            self.access_token = f.readline().strip('\n').strip()

    def write_config(self):
        if not os.path.exists(self.path_to_configdir):
            os.makedirs(self.path_to_configdir)

        profiles = {}
        new_config = {
            "host": to_str(self.host),
            "port": self.port,
            "username": to_str(self.username),
            "password": to_str(self.password),
        }
        try:
            with codecs.open(self.path_to_configfile, 'r', 'utf-8') as f:
                profiles = yaml.load(''.join(f.readlines()))
                profiles[self.profile] = new_config
        except FileNotFoundError:
            profiles[self.profile] = new_config

        with codecs.open(self.path_to_configfile, 'w', 'utf-8') as f:
            f.writelines(yaml.safe_dump(profiles, default_flow_style=False))

    def write_access_token(self):
        if not os.path.exists(self.path_to_configdir):
            os.makedirs(self.path_to_configdir)

        with codecs.open(self.path_to_accesstoken, 'w', 'utf-8') as f:
            f.write(self.access_token)

    @property
    def port(self):
        return int(self._port)

    @port.setter
    def port(self, _port):
        self._port = int(_port)

    @property
    def api_uri(self):
        return "http://{}:{}".format(self.host, self.port)


from setuptools import setup, find_packages

setup(
    name='paicli',
    version="0.5.1",
    description='Client for PAI',
    author='Sotetsu KOYAMADA',
    url='',
    author_email='koyamada-s@sys.i.kyoto-u.ac.jp',
    license='MIT',
    install_requires=["requests",
                      "pyyaml",
                      "prompt_toolkit==1.0.9",
                      "prettytable",
                      "termcolor",
                      "colorama",
                      "click",
                      ],
    packages=find_packages(),
    entry_points={
        'console_scripts': 'pai = paicli.main:main'
    },
    classifiers=[
        "Programming Language :: Python :: 3.5",
        "License :: OSI Approved :: MIT License"
    ]
)
